<?php

/**
 * The LaaS API Service class.
 *
 * @package    Easy_Translate
 * @subpackage Easy_Translate/admin
 */
class Easy_Translate_API_Laas extends Easy_Translate_API_Service
{
    const SERVICE_PREFIX = 'laas';

    /**
     * Send text to translate.
     *
     * @param string      $source
     * @param array       $target
     * @param array       $content
     * @param string      $workflow (translation, self+machine_translation)
     * @param string|null $name
     * @return array|string[]
     */
    public function translate(
        string $source,
        array $target,
        array $content,
        string $workflow = 'translation',
        string $name = null
    ) {
        $source = $this->normalize_source($source);
        $target = $this->normalize_target($target);

        $path = $this->team_path('projects');
        $siteUrl = get_site_url();

        if ($name === null) {
            $urlParts = parse_url(home_url());
            $hostName = $urlParts['host'] ?? $siteUrl;
            $name = '[' . date('Y-m-d') . '] ' . $hostName . ' translations';
        }

        $body = wp_json_encode([
            'data' => [
                'type'       => 'project',
                'attributes' => [
                    'name'             => $name,
                    'source_language'  => $source,
                    'target_languages' => $target,
                    'content'          => $content,
                    'workflow'         => $workflow,
                    'callback_url'     => "{$siteUrl}/wp-admin/admin-ajax.php?action=" . self::WEB_HOOK_ACTION,
                ],
            ],
        ]);
        $response = $this->wp_remote_post(
            $path,
            ['body' => $body, 'headers' => ['Content-Type' => 'application/json']]
        );
        $http_code = wp_remote_retrieve_response_code($response);
        $body = $this->decode_body($response);

        if ($http_code != 201) {
            $error = isset($body['message']) && $body['message'] ? $body['message'] : __('Error', EASY_TRANSLATE);
            $extra = isset($body['errors']) && $body['errors'] ? ' ' . json_encode($body['errors']) : '';

            return [
                'error' => $error . $extra,
            ];
        }

        return $body;
    }

    /**
     * Get the projects belonging to the User.
     *
     * @param string $params
     * @return mixed
     */
    public function fetch_all_projects(string $params = ''): array
    {
        $response = $this->wp_remote_get($this->team_path("projects{$params}"));
        $http_code = wp_remote_retrieve_response_code($response);
        $body = $this->decode_body($response);

        if ($http_code != 200) {
            return [
                'error' => isset($body['message']) && $body['message'] != '' ? $body['message'] : __('Error',
                    EASY_TRANSLATE),
                'hint'  => isset($body['hint']) && $body['hint'] ? $body['hint'] : __('Please verify your authentication status',
                    EASY_TRANSLATE),
            ];
        }

        return $body;
    }

    /**
     * Get the Task final processed (eg: translated) content.
     *
     * @param string $path
     * @return array
     */
    public function get_target_content(string $path): array
    {
        $response = $this->wp_remote_get_raw($path);

        $http_code = wp_remote_retrieve_response_code($response);
        $body = $this->decode_body($response);

        if ($http_code != 200) {
            return [
                'error' => isset($body['message']) && $body['message'] ? $body['message'] : __('Error', EASY_TRANSLATE),
            ];
        }

        return $body;
    }

    /**
     * Accept Price for Project.
     *
     * @param string $projectId
     * @return mixed
     */
    public function accept_project(string $projectId)
    {
        $path = $this->team_path("projects/{$projectId}/accept-price");
        $response = $this->wp_remote_post($path, ['headers' => ['Content-Type' => 'application/json']]);
        $httpCode = wp_remote_retrieve_response_code($response);
        $body = $this->decode_body($response);

        if ($httpCode != 200) {
            return [
                'error' => isset($body['message']) && $body['message'] != '' ? $body['message'] : __('Error',
                    EASY_TRANSLATE),
                'hint'  => isset($body['hint']) && $body['hint'] ? $body['hint'] : __('Please verify your authentication status',
                    EASY_TRANSLATE),
            ];
        }

        return $body;
    }

    /**
     * Decline Price for Project.
     *
     * @param string $projectId
     * @return array
     */
    public function decline_project(string $projectId): array
    {
        $path = $this->team_path("projects/{$projectId}/decline-price");
        $response = $this->wp_remote_post($path, ['headers' => ['Content-Type' => 'application/json']]);
        $httpCode = wp_remote_retrieve_response_code($response);
        $body = $this->decode_body($response);

        if ($httpCode != 200) {
            return [
                'error' => isset($body['message']) && $body['message'] != '' ? $body['message'] : __('Error',
                    EASY_TRANSLATE),
                'hint'  => isset($body['hint']) && $body['hint'] ? $body['hint'] : __('Please verify your authentication status',
                    EASY_TRANSLATE),
            ];
        }

        return $body;
    }

    protected function get_service_prefix(): string
    {
        return self::SERVICE_PREFIX;
    }
}
