<?php

/**
 * The String Library API Service class.
 *
 * @package    Easy_Translate
 * @subpackage Easy_Translate/admin
 */
class Easy_Translate_API_String_Library extends Easy_Translate_API_Service
{
    const SERVICE_PREFIX = 'strings-library';

    public function sync_to_string_library(string $sourceLanguage, array $targetLanguages, array $content)
    {
        if (is_null($stringLibraryId = $this->get_string_library_id())) {
            return ['error' => 'String Library ID is not set'];
        }
        $keys = [];
        foreach ($content as $string) {
            $sourceLanguage = $this->normalize_source($string['source_language']);
            $key = $this->buildKey($string['name'], $string['context']);
            $keys[$key]['name'] = $key;
            $keys[$key]['external_id'] = $string['source_id'];
            $keys[$key]['strings'][$sourceLanguage] = [
                'text'          => $string['source_text'],
                'language_code' => $sourceLanguage,
                'external_id'   => $string['source_id'],
            ];
            if (!is_null($string['target_id'])) {
                $targetLanguage = $this->normalize_target($string['target_language']);
                $keys[$key]['strings'][$targetLanguage] = [
                    'text'          => $string['target_text'],
                    'language_code' => $targetLanguage,
                    'external_id'   => $string['target_id'],
                ];
            }
        }

        $body = wp_json_encode([
            'data' => [
                'type'       => 'libraries',
                'attributes' => [
                    'source_language'  => $this->normalize_source($sourceLanguage),
                    'target_languages' => $this->normalize_target($targetLanguages),
                    'keys'             => $keys,
                ],
            ],
        ]);

        $response = $this->wp_remote_post(
            $this->team_path("libraries/{$stringLibraryId}/sync"),
            ['body' => $body, 'headers' => ['Content-Type' => 'application/json']]
        );
        $httpCode = wp_remote_retrieve_response_code($response);
        $responseBody = $this->decode_body($response);

        if ($httpCode != 200) {
            $error = $responseBody['message'] ?? $responseBody[0] ?? __('Error', EASY_TRANSLATE);
            $extra = isset($responseBody['errors']) && $responseBody['errors'] ? ' ' . json_encode($responseBody['errors']) : '';

            return [
                'error' => $error . $extra,
            ];
        }

        return $responseBody ?? [];
    }

    public function buildKey(string $name, string $context): string
    {
        $name = str_replace(' ', '-', $name);

        return strtolower("{$context}.{$name}");
    }

    /**
     * @param array $targetLanguages
     * @param array $keys
     * @return Generator|array
     */
    public function auto_paginated_translations(array $targetLanguages, array $keys = [])
    {
        $targetLanguages = $this->normalize_target($targetLanguages);
        $currentPage = 1;
        $response = $this->get_library_translations($targetLanguages, $keys, $currentPage, 500);
        if ($response['error'] ?? false) {
            return $response;
        }

        while (true) {
            $data = $response['data'] ?? [];

            foreach ($data as $translation) {
                yield $translation;
            }
            if (empty($data) || $response['meta']['last_page'] <= $currentPage) {
                break;
            }

            $currentPage += 1;
            $response = $this->get_library_translations($targetLanguages, $keys, $currentPage, 500);
        }
    }

    protected function get_library_translations(
        array $targetLanguages,
        array $keys = [],
        int $page = 1,
        int $perPage = 100
    ): array {
        if (is_null($stringLibraryId = $this->get_string_library_id())) {
            return ['error' => 'String Library ID is not set'];
        }
        $query = [
            'filters' => ['target_languages' => $targetLanguages],
            'page'    => $page,
            'perPage' => $perPage,
        ];
        if (!empty($keys)) {
            $query['filters']['keys'] = $keys;
        }
        $queryString = http_build_query($query);

        $response = $this->wp_remote_get(
            "{$this->team_path("libraries/{$stringLibraryId}/bilingual")}?{$queryString}"
        );
        $httpCode = wp_remote_retrieve_response_code($response);
        $body = $this->decode_body($response);
        if ($httpCode != 200) {
            return [
                'error' => isset($body['message']) && $body['message'] ? $body['message'] : __('Error', EASY_TRANSLATE),
            ];
        }

        return $body;
    }

    protected function get_service_prefix(): string
    {
        return self::SERVICE_PREFIX;
    }
}
